@@ -86,6 +86,9 @@ typedef struct {
 	struct pw_stream *stream;
 	struct spa_hook stream_listener;
 
+	struct pw_time time;
+	struct spa_io_rate_match *rate_match;
+
 	struct spa_audio_info_raw format;
 } snd_pcm_pipewire_t;
 
@@ -178,8 +181,33 @@ static snd_pcm_sframes_t snd_pcm_pipewire_pointer(snd_pcm_ioplug_t *io)
 #endif
 }
 
+static int snd_pcm_pipewire_delay(snd_pcm_ioplug_t *io, snd_pcm_sframes_t *delayp)
+{
+	snd_pcm_pipewire_t *pw = io->private_data;
+	int64_t elapsed = 0, filled;
+
+	if (pw->time.rate.num != 0) {
+		struct timespec ts;
+		int64_t diff;
+		clock_gettime(CLOCK_MONOTONIC, &ts);
+		diff = SPA_TIMESPEC_TO_NSEC(&ts) - pw->time.now;
+	        elapsed = (pw->time.rate.denom * diff) / (pw->time.rate.num * SPA_NSEC_PER_SEC);
+		if (elapsed > pw->time.delay)
+			elapsed = pw->time.delay;
+	}
+	filled = pw->time.delay + snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
+
+	if (io->stream == SND_PCM_STREAM_PLAYBACK)
+		*delayp = filled - SPA_MIN(elapsed, filled);
+	else
+		*delayp = filled + elapsed;
+
+	return 0;
+}
+
 static int
-snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uframes_t *hw_avail)
+snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b,
+		snd_pcm_uframes_t *hw_avail,snd_pcm_uframes_t want)
 {
 	snd_pcm_ioplug_t *io = &pw->io;
 	snd_pcm_channel_area_t *pwareas;
@@ -199,10 +227,12 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uf
 	} else {
 		nframes = d[0].chunk->size / pw->stride;
 	}
+	want = SPA_MIN(nframes, want);
+	nframes = SPA_MIN(want, *hw_avail);
 
 	if (pw->blocks == 1) {
 		if (io->stream == SND_PCM_STREAM_PLAYBACK) {
-			d[0].chunk->size = nframes * pw->stride;
+			d[0].chunk->size = want * pw->stride;
 			d[0].chunk->offset = 0;
 		}
 		ptr = SPA_MEMBER(d[0].data, d[0].chunk->offset, void);
@@ -214,7 +244,7 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uf
 	} else {
 		for (channel = 0; channel < io->channels; channel++) {
 			if (io->stream == SND_PCM_STREAM_PLAYBACK) {
-				d[channel].chunk->size = nframes * pw->stride;
+				d[channel].chunk->size = want * pw->stride;
 				d[channel].chunk->offset = 0;
 			}
 			ptr = SPA_MEMBER(d[channel].data, d[channel].chunk->offset, void);
@@ -227,7 +257,7 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uf
 	if (io->state == SND_PCM_STATE_RUNNING ||
 		io->state == SND_PCM_STATE_DRAINING) {
 		snd_pcm_uframes_t hw_ptr = pw->hw_ptr;
-		xfer = SPA_MIN(nframes, *hw_avail);
+		xfer = nframes;
 		if (xfer > 0) {
 			const snd_pcm_channel_area_t *areas = snd_pcm_ioplug_mmap_areas(io);
 			const snd_pcm_uframes_t offset = hw_ptr % io->buffer_size;
@@ -253,13 +283,13 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uf
 		}
 	}
 	/* check if requested frames were copied */
-	if (xfer < nframes) {
-		/* always fill the not yet written JACK buffer with silence */
+	if (xfer < want) {
+		/* always fill the not yet written PipeWire buffer with silence */
 		if (io->stream == SND_PCM_STREAM_PLAYBACK) {
-			const snd_pcm_uframes_t frames = nframes - xfer;
+			const snd_pcm_uframes_t frames = want - xfer;
 
 			snd_pcm_areas_silence(pwareas, xfer, io->channels,
-								  frames, io->format);
+							  frames, io->format);
 		}
 		if (io->state == SND_PCM_STATE_RUNNING ||
 			io->state == SND_PCM_STATE_DRAINING) {
@@ -267,7 +297,6 @@ snd_pcm_pipewire_process(snd_pcm_pipewire_t *pw, struct pw_buffer *b, snd_pcm_uf
 			pw->xrun_detected = true;
 		}
 	}
-
 	return 0;
 }
 
@@ -303,12 +332,26 @@ static void on_stream_param_changed(void *data, uint32_t id, const struct spa_po
 	pw_stream_update_params(pw->stream, params, n_params);
 }
 
+static void on_stream_io_changed(void *data, uint32_t id, void *area, uint32_t size)
+{
+	snd_pcm_pipewire_t *pw = data;
+	switch (id) {
+	case SPA_IO_RateMatch:
+		pw->rate_match = area;
+		break;
+	default:
+		break;
+	}
+}
+
 static void on_stream_process(void *data)
 {
 	snd_pcm_pipewire_t *pw = data;
 	snd_pcm_ioplug_t *io = &pw->io;
 	struct pw_buffer *b;
-	snd_pcm_uframes_t hw_avail;
+	snd_pcm_uframes_t hw_avail, want;
+
+	pw_stream_get_time(pw->stream, &pw->time);
 
 	hw_avail = snd_pcm_ioplug_hw_avail(io, pw->hw_ptr, io->appl_ptr);
 
@@ -321,7 +364,10 @@ static void on_stream_process(void *data)
 	if (b == NULL)
 		return;
 
-	snd_pcm_pipewire_process(pw, b, &hw_avail);
+	want = pw->rate_match ? pw->rate_match->size : hw_avail;
+	pw_log_trace(NAME" %p: avail:%lu want:%lu", pw, hw_avail, want);
+
+	snd_pcm_pipewire_process(pw, b, &hw_avail, want);
 
 	pw_stream_queue_buffer(pw->stream, b);
 
@@ -345,6 +391,7 @@ static void on_stream_drained(void *data)
 static const struct pw_stream_events stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.param_changed = on_stream_param_changed,
+	.io_changed = on_stream_io_changed,
 	.process = on_stream_process,
 	.drained = on_stream_drained,
 };
@@ -741,6 +788,7 @@ static snd_pcm_ioplug_callback_t pipewire_pcm_callback = {
 	.stop = snd_pcm_pipewire_stop,
 	.pause = snd_pcm_pipewire_pause,
 	.pointer = snd_pcm_pipewire_pointer,
+	.delay = snd_pcm_pipewire_delay,
 	.drain = snd_pcm_pipewire_drain,
 	.prepare = snd_pcm_pipewire_prepare,
 	.poll_revents = snd_pcm_pipewire_poll_revents,
