@@ -275,6 +275,7 @@ static const struct spa_dict_item node_info_items[] = {
 	{ SPA_KEY_DEVICE_API, "alsa" },
 	{ SPA_KEY_MEDIA_CLASS, "Audio/Source" },
 	{ SPA_KEY_NODE_DRIVER, "true" },
+	{ SPA_KEY_NODE_PAUSE_ON_IDLE, "false" },
 };
 
 static void emit_node_info(struct state *this, bool full)
@@ -816,6 +817,10 @@ impl_init(const struct spa_handle_factory *factory,
 			}
 		} else if (!strcmp(info->items[i].key, "api.alsa.period-size")) {
 			this->default_period_size = atoi(info->items[i].value);
+		} else if (!strcmp(info->items[i].key, "api.alsa.headroom")) {
+			this->default_headroom = atoi(info->items[i].value);
+		} else if (!strcmp(info->items[i].key, "api.alsa.disable-mmap")) {
+			this->disable_mmap = atoi(info->items[i].value);
 		}
 	}
 	return 0;
