@@ -111,10 +111,12 @@ struct state {
 	struct spa_audio_info current_format;
 
 	uint32_t default_period_size;
+	uint32_t default_headroom;
 	uint32_t default_format;
 	unsigned int default_channels;
 	unsigned int default_rate;
 	struct channel_map default_pos;
+	unsigned int disable_mmap;
 
 	snd_pcm_uframes_t buffer_frames;
 	snd_pcm_uframes_t period_frames;
@@ -159,6 +161,8 @@ struct state {
 	unsigned int following:1;
 	unsigned int matching:1;
 	unsigned int resample:1;
+	unsigned int use_mmap:1;
+	unsigned int planar:1;
 
 	int64_t sample_count;
 
@@ -186,7 +190,7 @@ int spa_alsa_reassign_follower(struct state *state);
 int spa_alsa_pause(struct state *state);
 int spa_alsa_close(struct state *state);
 
-int spa_alsa_write(struct state *state, snd_pcm_uframes_t silence);
+int spa_alsa_write(struct state *state);
 int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence);
 
 void spa_alsa_recycle_buffer(struct state *state, uint32_t buffer_id);
