@@ -666,7 +666,7 @@ static int do_start(struct impl *this)
 	this->codec_data = this->codec->init(this->codec, 0,
 			this->transport->configuration,
 			this->transport->configuration_len,
-			&port->current_format,
+			&port->current_format, NULL,
 			this->transport->write_mtu);
 	if (this->codec_data == NULL)
 		return -EIO;
